@@ -41,8 +41,8 @@ static int initialized;
 static int module;
 
 /* This value is used by DSA and RSA checks in addition to the hard
- * coded length checks.  It allows to increase the required key length
- * using a confue file.  */
+ * coded length checks.  It allows one to increase the required key length
+ * using a config file.  */
 static unsigned int min_compliant_rsa_length;
 
 /* Return the address of a compliance cache variable for COMPLIANCE.
@@ -139,7 +139,7 @@ gnupg_pk_is_compliant (enum gnupg_compliance_mode compliance, int algo,
 		       gcry_mpi_t key[], unsigned int keylength,
                        const char *curvename)
 {
-  enum { is_rsa, is_dsa, is_elg, is_ecc } algotype;
+  enum { is_rsa, is_dsa, is_elg, is_ecc, is_kem } algotype;
   int result = 0;
 
   if (! initialized)
@@ -173,6 +173,10 @@ gnupg_pk_is_compliant (enum gnupg_compliance_mode compliance, int algo,
     case PUBKEY_ALGO_ELGAMAL:
       return 0; /* Signing with Elgamal is not at all supported.  */
 
+    case PUBKEY_ALGO_KYBER:
+      algotype = is_kem;
+      break;
+
     default: /* Unknown.  */
       return 0;
     }
@@ -227,6 +231,10 @@ gnupg_pk_is_compliant (enum gnupg_compliance_mode compliance, int algo,
                         || !strcmp (curvename, "brainpoolP512r1")));
           break;
 
+        case is_kem:
+          result = 0;
+          break;
+
         default:
           result = 0;
         }
