@@ -31,7 +31,7 @@
 #include "certcache.h"
 #include "ocsp.h"
 
-/* The maximum size we allow as a response from an OCSP reponder. */
+/* The maximum size we allow as a response from an OCSP responder. */
 #define MAX_RESPONSE_SIZE 65536
 
 
@@ -227,7 +227,7 @@ do_ocsp_request (ctrl_t ctrl, ksba_ocsp_t ocsp,
             case 301:
             case 302:
               {
-                const char *s = http_get_header (http, "Location");
+                const char *s = http_get_header (http, "Location", 0);
 
                 log_info (_("URL '%s' redirected to '%s' (%u)\n"),
                           url, s?s:"[none]", http_get_status_code (http));
@@ -526,7 +526,7 @@ check_signature_core (ctrl_t ctrl, ksba_cert_t cert, gcry_sexp_t s_sig,
 /* Check the signature of an OCSP response.  OCSP is the context,
    S_SIG the signature value and MD the handle of the hash we used for
    the response.  This function automagically finds the correct public
-   key.  If SIGNER_FPR_LIST is not NULL, the default OCSP reponder has been
+   key.  If SIGNER_FPR_LIST is not NULL, the default OCSP responder has been
    used and thus the certificate is one of those identified by
    the fingerprints. */
 static gpg_error_t
@@ -651,7 +651,7 @@ check_signature (ctrl_t ctrl,
    or directly through the CERT object is valid by running an OCSP
    transaction.  With FORCE_DEFAULT_RESPONDER set only the configured
    default responder is used.  If R_REVOKED_AT or R_REASON are not
-   NULL and the certificat has been revoked the revocation time and
+   NULL and the certificate has been revoked the revocation time and
    the reasons are stored there. */
 gpg_error_t
 ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr,
@@ -723,7 +723,7 @@ ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr,
     }
 
   /* Figure out the OCSP responder to use.
-     1. Try to get the reponder from the certificate.
+     1. Try to get the responder from the certificate.
         We do only take http and https style URIs into account.
      2. If this fails use the default responder, if any.
    */
