@@ -117,6 +117,7 @@ enum cmd_and_opt_values {
   oLogTime,
 
   oEnableSpecialFilenames,
+  oDisableFdTranslation,
 
   oAgentProgram,
   oDisplay,
@@ -216,7 +217,10 @@ enum cmd_and_opt_values {
   oCompatibilityFlags,
   oKbxBufferSize,
   oAlwaysTrust,
-  oNoAutostart
+  oNoAutostart,
+  oAssertSigner,
+
+  oNoop
  };
 
 
@@ -310,6 +314,7 @@ static gpgrt_opt_t opts[] = {
                 N_("|FILE|take policy information from FILE")),
   ARGPARSE_s_s (oCompliance, "compliance",   "@"),
   ARGPARSE_p_u (oMinRSALength, "min-rsa-length", "@"),
+  ARGPARSE_s_s (oAssertSigner, "assert-signer", "@"),
   ARGPARSE_s_n (oNoCommonCertsImport, "no-common-certs-import", "@"),
   ARGPARSE_s_s (oIgnoreCertExtension, "ignore-cert-extension", "@"),
   ARGPARSE_s_s (oIgnoreCertWithOID, "ignore-cert-with-oid", "@"),
@@ -428,6 +433,7 @@ static gpgrt_opt_t opts[] = {
   ARGPARSE_s_n (oAnswerNo,  "no",  N_("assume no on most questions")),
   ARGPARSE_s_i (oStatusFD, "status-fd", N_("|FD|write status info to this FD")),
   ARGPARSE_s_n (oEnableSpecialFilenames, "enable-special-filenames", "@"),
+  ARGPARSE_s_n (oDisableFdTranslation, "disable-fd-translation", "@"),
   ARGPARSE_s_i (oPassphraseFD,    "passphrase-fd", "@"),
   ARGPARSE_s_s (oPinentryMode,    "pinentry-mode", "@"),
 
@@ -500,6 +506,9 @@ static struct compatibility_flags_s compatibility_flags [] =
 
 /* Global variable to keep an error count. */
 int gpgsm_errors_seen = 0;
+/* If opt.assert_signer_list is used and this variable is not true
+ * gpg will be forced to return EXIT_FAILURE.  */
+int assert_signer_true = 0;
 
 /* It is possible that we are currentlu running under setuid permissions */
 static int maybe_setuid = 1;
@@ -537,7 +546,6 @@ static void set_cmd (enum cmd_and_opt_values *ret_cmd,
                      enum cmd_and_opt_values new_cmd );
 
 static void emergency_cleanup (void);
-static int open_read (const char *filename);
 static estream_t open_es_fread (const char *filename, const char *mode);
 static estream_t open_es_fwrite (const char *filename);
 static void run_protect_tool (int argc, char **argv);
@@ -599,10 +607,6 @@ our_md_test_algo (int algo)
 }
 
 
-/* nPth wrapper function definitions. */
-ASSUAN_SYSTEM_NPTH_IMPL;
-
-
 static char *
 make_libversion (const char *libname, const char *(*getfnc)(const char*))
 {
@@ -1029,12 +1033,12 @@ main ( int argc, char **argv)
   /* Set the default policy file */
   opt.policy_file = make_filename (gnupg_homedir (), "policies.txt", NULL);
 
-  /* The configuraton directories for use by gpgrt_argparser.  */
+  /* The configuration directories for use by gpgrt_argparser.  */
   gpgrt_set_confdir (GPGRT_CONFDIR_SYS, gnupg_sysconfdir ());
   gpgrt_set_confdir (GPGRT_CONFDIR_USER, gnupg_homedir ());
 
   /* We are re-using the struct, thus the reset flag.  We OR the
-   * flags so that the internal intialized flag won't be cleared. */
+   * flags so that the internal initialized flag won't be cleared. */
   argc        = orig_argc;
   argv        = orig_argv;
   pargs.argc  = &argc;
@@ -1466,6 +1470,10 @@ main ( int argc, char **argv)
           enable_special_filenames ();
           break;
 
+        case oDisableFdTranslation:
+          disable_translate_sys2libc_fd ();
+          break;
+
         case oValidationModel: parse_validation_model (pargs.r.ret_str); break;
 
 	case oKeyServer:
@@ -1517,6 +1525,12 @@ main ( int argc, char **argv)
           keybox_set_buffersize (pargs.r.ret_ulong, 0);
           break;
 
+        case oAssertSigner:
+          add_to_strlist (&opt.assert_signer_list, pargs.r.ret_str);
+          break;
+
+        case oNoop: break;
+
         default:
           if (configname)
             pargs.err = ARGPARSE_PRINT_WARNING;
@@ -1611,8 +1625,8 @@ main ( int argc, char **argv)
 
 
   npth_init ();
-  assuan_set_system_hooks (ASSUAN_SYSTEM_NPTH);
   gpgrt_set_syscall_clamp (npth_unprotect, npth_protect);
+  assuan_control (ASSUAN_CONTROL_REINIT_SYSCALL_CLAMP, NULL);
 
 
 /*   if (opt.qualsig_approval && !opt.quiet) */
@@ -1801,7 +1815,7 @@ main ( int argc, char **argv)
             {
               log_info (_("importing common certificates '%s'\n"),
                         filelist[0]);
-              gpgsm_import_files (&ctrl, 1, filelist, open_read);
+              gpgsm_import_files (&ctrl, 1, filelist, open_es_fread);
             }
           xfree (filelist[0]);
         }
@@ -1872,7 +1886,7 @@ main ( int argc, char **argv)
       /* We do not require a recipient for decryption but because
        * recipients and signers are always checked and log_error is
        * sometimes used (for failed signing keys or due to a failed
-       * CRL checking) that would have bumbed up the error counter.
+       * CRL checking) that would have bumped up the error counter.
        * We clear the counter in the decryption case because there is
        * no reason to force decryption to fail. */
       if (cmd == aDecrypt && !errcount)
@@ -1939,9 +1953,20 @@ main ( int argc, char **argv)
         set_binary (stdin);
 
         if (!argc) /* Source is stdin. */
-          err = gpgsm_encrypt (&ctrl, recplist, 0, fp);
+          err = gpgsm_encrypt (&ctrl, recplist, es_stdin, fp);
         else if (argc == 1)  /* Source is the given file. */
-          err = gpgsm_encrypt (&ctrl, recplist, open_read (*argv), fp);
+          {
+            estream_t data_fp = es_fopen (*argv, "rb");
+
+            if (!data_fp)
+              {
+                log_error (_("can't open '%s': %s\n"), *argv,
+                           strerror (errno));
+                gpgsm_exit (2);
+              }
+            err = gpgsm_encrypt (&ctrl, recplist, data_fp, fp);
+            es_fclose (data_fp);
+          }
         else
           wrong_args ("--encrypt [datafile]");
 
@@ -1960,10 +1985,20 @@ main ( int argc, char **argv)
            signing because that is what gpg does.*/
         set_binary (stdin);
         if (!argc) /* Create from stdin. */
-          err = gpgsm_sign (&ctrl, signerlist, 0, detached_sig, fp);
+          err = gpgsm_sign (&ctrl, signerlist, es_stdin, detached_sig, fp);
         else if (argc == 1) /* From file. */
-          err = gpgsm_sign (&ctrl, signerlist,
-                      open_read (*argv), detached_sig, fp);
+          {
+            estream_t data_fp = es_fopen (*argv, "rb");
+
+            if (!data_fp)
+              {
+                log_error (_("can't open '%s': %s\n"), *argv,
+                           strerror (errno));
+                gpgsm_exit (2);
+              }
+            err = gpgsm_sign (&ctrl, signerlist, data_fp, detached_sig, fp);
+            es_fclose (data_fp);
+          }
         else
           wrong_args ("--sign [datafile]");
 
@@ -2004,11 +2039,43 @@ main ( int argc, char **argv)
           fp = open_es_fwrite (opt.outfile);
 
         if (!argc)
-          gpgsm_verify (&ctrl, 0, -1, fp); /* normal signature from stdin */
+          /* normal signature from stdin */
+          gpgsm_verify (&ctrl, es_stdin, NULL, fp);
         else if (argc == 1)
-          gpgsm_verify (&ctrl, open_read (*argv), -1, fp); /* std signature */
+          {
+            estream_t in_fp = es_fopen (*argv, "rb");
+
+            if (!in_fp)
+              {
+                log_error (_("can't open '%s': %s\n"), *argv,
+                           strerror (errno));
+                gpgsm_exit (2);
+              }
+            gpgsm_verify (&ctrl, in_fp, NULL, fp); /* std signature */
+            es_fclose (in_fp);
+          }
         else if (argc == 2) /* detached signature (sig, detached) */
-          gpgsm_verify (&ctrl, open_read (*argv), open_read (argv[1]), NULL);
+          {
+            estream_t in_fp = es_fopen (*argv, "rb");
+            estream_t data_fp = es_fopen (argv[1], "rb");
+
+            if (!in_fp)
+              {
+                log_error (_("can't open '%s': %s\n"), *argv,
+                           strerror (errno));
+                gpgsm_exit (2);
+              }
+            if (!data_fp)
+              {
+                log_error (_("can't open '%s': %s\n"), argv[1],
+                           strerror (errno));
+                gpgsm_exit (2);
+              }
+
+            gpgsm_verify (&ctrl, in_fp, data_fp, NULL);
+            es_fclose (in_fp);
+            es_fclose (data_fp);
+          }
         else
           wrong_args ("--verify [signature [detached_data]]");
 
@@ -2022,9 +2089,19 @@ main ( int argc, char **argv)
 
         set_binary (stdin);
         if (!argc)
-          err = gpgsm_decrypt (&ctrl, 0, fp); /* from stdin */
+          err = gpgsm_decrypt (&ctrl, es_stdin, fp); /* from stdin */
         else if (argc == 1)
-          err = gpgsm_decrypt (&ctrl, open_read (*argv), fp); /* from file */
+          {
+            estream_t data_fp = es_fopen (*argv, "rb");
+            if (!data_fp)
+              {
+                log_error (_("can't open '%s': %s\n"), *argv,
+                           strerror (errno));
+                gpgsm_exit (2);
+              }
+            err = gpgsm_decrypt (&ctrl, data_fp, fp); /* from file */
+            es_fclose (data_fp);
+          }
         else
           wrong_args ("--decrypt [filename]");
 
@@ -2115,7 +2192,7 @@ main ( int argc, char **argv)
 
 
     case aImport:
-      gpgsm_import_files (&ctrl, argc, argv, open_read);
+      gpgsm_import_files (&ctrl, argc, argv, open_es_fread);
       break;
 
     case aExport:
@@ -2265,6 +2342,15 @@ emergency_cleanup (void)
 void
 gpgsm_exit (int rc)
 {
+  if (rc)
+    ;
+  else if (log_get_errorcount(0))
+    rc = 2;
+  else if (gpgsm_errors_seen)
+    rc = 1;
+  else if (opt.assert_signer_list && !assert_signer_true)
+    rc = 1;
+
   gcry_control (GCRYCTL_UPDATE_RANDOM_SEED_FILE);
   if (opt.debug & DBG_MEMSTAT_VALUE)
     {
@@ -2274,7 +2360,6 @@ gpgsm_exit (int rc)
   if (opt.debug)
     gcry_control (GCRYCTL_DUMP_SECMEM_STATS );
   emergency_cleanup ();
-  rc = rc? rc : log_get_errorcount(0)? 2 : gpgsm_errors_seen? 1 : 0;
   exit (rc);
 }
 
@@ -2316,49 +2401,24 @@ gpgsm_parse_validation_model (const char *model)
 }
 
 
-
-/* Open the FILENAME for read and return the file descriptor.  Stop
-   with an error message in case of problems.  "-" denotes stdin and
-   if special filenames are allowed the given fd is opened instead.  */
-static int
-open_read (const char *filename)
-{
-  int fd;
-
-  if (filename[0] == '-' && !filename[1])
-    {
-      set_binary (stdin);
-      return 0; /* stdin */
-    }
-  fd = check_special_filename (filename, 0, 0);
-  if (fd != -1)
-    return fd;
-  fd = gnupg_open (filename, O_RDONLY | O_BINARY, 0);
-  if (fd == -1)
-    {
-      log_error (_("can't open '%s': %s\n"), filename, strerror (errno));
-      gpgsm_exit (2);
-    }
-  return fd;
-}
-
 /* Same as open_read but return an estream_t.  */
 static estream_t
 open_es_fread (const char *filename, const char *mode)
 {
-  int fd;
+  gnupg_fd_t fd;
   estream_t fp;
 
   if (filename[0] == '-' && !filename[1])
-    fd = fileno (stdin);
+    return es_fpopen_nc (stdin, mode);
   else
-    fd = check_special_filename (filename, 0, 0);
-  if (fd != -1)
+    fd = gnupg_check_special_filename (filename);
+  if (fd != GNUPG_INVALID_FD)
     {
-      fp = es_fdopen_nc (fd, mode);
+      fp = open_stream_nc (fd, mode);
       if (!fp)
         {
-          log_error ("es_fdopen(%d) failed: %s\n", fd, strerror (errno));
+          log_error ("es_fdopen(%d) failed: %s\n", FD_DBG (fd),
+                     strerror (errno));
           gpgsm_exit (2);
         }
       return fp;
@@ -2380,23 +2440,24 @@ open_es_fread (const char *filename, const char *mode)
 static estream_t
 open_es_fwrite (const char *filename)
 {
-  int fd;
+  gnupg_fd_t fd;
   estream_t fp;
 
   if (filename[0] == '-' && !filename[1])
     {
       fflush (stdout);
-      fp = es_fdopen_nc (fileno(stdout), "wb");
+      fp = es_fpopen_nc (stdout, "wb");
       return fp;
     }
 
-  fd = check_special_filename (filename, 1, 0);
-  if (fd != -1)
+  fd = gnupg_check_special_filename (filename);
+  if (fd != GNUPG_INVALID_FD)
     {
-      fp = es_fdopen_nc (fd, "wb");
+      fp = open_stream_nc (fd, "wb");
       if (!fp)
         {
-          log_error ("es_fdopen(%d) failed: %s\n", fd, strerror (errno));
+          log_error ("es_fdopen(%d) failed: %s\n",
+                     FD_DBG (fd), strerror (errno));
           gpgsm_exit (2);
         }
       return fp;
